==> robogame_engine-0.7.4.tar.gz: robogame_engine-0.7.4/robogame_engine/states.py <==
# -*- coding: utf-8 -*-
from robogame_engine.theme import theme
from robogame_engine.events import EventStoppedAtTargetPoint
from robogame_engine.geometry import normalise_angle, Vector
from robogame_engine.utils import CanLogging


class ObjectState(CanLogging):

    def __init__(self, obj, target=None, speed=None, **kwargs):
        self.obj = obj
        self.kwargs = kwargs
        self.target = target
        self.speed = theme.MAX_SPEED if speed is None else speed
        target_coord = target.coord if hasattr(target, 'coord') else target
        if self.target:
            self.vector = Vector.from_points(self.obj.coord, target_coord, module=self.speed)
        else:
            self.vector = None

    def move(self, target, speed):
        if self.obj.rotate_mode == theme.ROTATE_TURNING:
            self.obj.state = StateTurning(obj=self.obj, target=target, speed=speed)
            self.obj.state.move_at_target = True
        else:
            self.obj.state = StateMoving(obj=self.obj, target=target, speed=speed)

    def stop(self):
        self.obj.state = StateStopped(obj=self.obj)

    def turn(self, vector, target):
        self.obj.state = StateTurning(obj=self.obj, vector=vector, target=target)

    def step(self):
        raise NotImplementedError

    def __str__(self):
        return "{}: {}".format(self.__class__.__name__, self.__dict__)


class StateTurning(ObjectState):
    move_at_target = False

    # def __init__(self, obj, target=None, speed=None, **kwargs):
    #     self.error("obj={obj}, target={target}, speed={speed}, kwargs={kwargs}", obj=obj, target=target, speed=speed, kwargs=kwargs)
    #     if target is None:
    #         self.warning("target is None!")
    #     super(StateTurning, self).__init__(obj, target, speed, **kwargs)

    def step(self):
        obj = self.obj
        delta = self.vector.direction - obj.direction
        if abs(delta) < theme.TURN_SPEED:
            obj.vector = self.vector
            if self.move_at_target:
                obj.state = StateMoving(obj=obj, target=self.target, speed=self.speed)
            else:
                obj.state = StateStopped(obj=obj)
        else:
            if -180 < delta < 0 or delta > 180:
                obj.vector.rotate(-theme.TURN_SPEED)
            else:
                obj.vector.rotate(theme.TURN_SPEED)


class StateMoving(ObjectState):

    def step(self):
        self.obj.coord += self.vector
        self.obj.vector = self.vector
        if self.obj.coord.near(self.target):
            self.obj.state = StateStopped(obj=self.obj)
            event = EventStoppedAtTargetPoint(self.target)
            self.obj.add_event(event)


class StateStopped(ObjectState):

    def stop(self):
        pass

    def step(self):
        pass


class StateDead(ObjectState):

    def move(self, target, speed):
        pass

    def stop(self):
        pass

    def turn(self, vector, target):
        pass

    def step(self):
        pass

==> robogame_engine-0.8.1.tar.gz: robogame_engine-0.8.1/robogame_engine/states.py <==
# -*- coding: utf-8 -*-

from .theme import theme
from .events import EventStoppedAtTargetPoint
from .geometry import Vector
from .utils import CanLogging


class ObjectState(CanLogging):

    def __init__(self, obj, target=None, speed=None, **kwargs):
        self.obj = obj
        self.kwargs = kwargs
        self.target_point = target.coord if hasattr(target, 'coord') else target
        self.speed = theme.MAX_SPEED if speed is None else speed
        if self.target_point:
            self.vector = Vector.from_points(self.obj.coord, self.target_point, module=self.speed)
        else:
            self.vector = None

    def move(self, target, speed):
        if self.obj.rotate_mode == theme.ROTATE_TURNING:
            self.obj.state = StateTurning(obj=self.obj, target=target, speed=speed)
            self.obj.state.move_at_target = True
        else:
            self.obj.state = StateMoving(obj=self.obj, target=target, speed=speed)

    def stop(self):
        self.obj.state = StateStopped(obj=self.obj)

    def turn(self, vector, target):
        self.obj.state = StateTurning(obj=self.obj, vector=vector, target=target)

    def step(self):
        raise NotImplementedError

    def __str__(self):
        return "{}: {}".format(self.__class__.__name__, self.__dict__)


class StateTurning(ObjectState):
    move_at_target = False

    def __init__(self, obj, target=None, speed=None, **kwargs):
        super(StateTurning, self).__init__(obj=obj, target=target, speed=speed, **kwargs)
        self.turn_speed = self.kwargs.get('turn_speed', theme.MAX_TURN_SPEED)

    def step(self):
        obj = self.obj
        delta = self.vector.direction - obj.direction
        if abs(delta) < self.turn_speed:
            obj.vector = self.vector
            if self.move_at_target:
                obj.state = StateMoving(obj=obj, target=self.target_point, speed=self.speed)
            else:
                obj.state = StateStopped(obj=obj)
        else:
            if -180 < delta < 0 or delta > 180:
                obj.vector.rotate(-self.turn_speed)
            else:
                obj.vector.rotate(self.turn_speed)


class StateMoving(ObjectState):

    def step(self):
        distance_to_target = self.obj.coord.distance_to(self.target_point)
        if distance_to_target < self.vector.module:
            self.obj.coord += Vector.from_direction(self.vector.direction, distance_to_target)
            self.obj.state = StateStopped(obj=self.obj)
            event = EventStoppedAtTargetPoint(self.target_point)
            self.obj.add_event(event)
        else:
            self.obj.coord += self.vector
            self.obj.vector = self.vector


class StateStopped(ObjectState):

    def stop(self):
        pass

    def step(self):
        pass


class StateDead(ObjectState):

    def move(self, target, speed):
        pass

    def stop(self):
        pass

    def turn(self, vector, target):
        pass

    def step(self):
        pass
